/**
 * Ex. 2: declare a synchronous channel and create two processes:
 * - the first process sends the letters 'a' through 'z' onto the channel
 * - the second process reads the values of the channel and outputs them
 * - check if sooner or later the second process will read the letter 'z'
 */

chan mqueue = [0] of { byte };

active proctype A() {
	byte x = 'a';
	do
		:: x <= 'z' ->
			atomic {
				printf("A: sending %c\n", x);
				mqueue!x;
			}
			x++;
		:: else ->
			break;
	od
	printf("A: terminated.\n");
}

active proctype B() {
	byte y;
	do
		:: atomic {
			mqueue?y ->
				printf("B: received %c\n", y);
			}
		:: y >= 'z' ->
			break;
	od;
	assert(y == 'z');
	printf("B: terminated.\n");
}

/*
 * NOTEs:
 * 1. atomic is necessary to bind the 'printf()' call to happen at the same
 *    time in which the process is able to send/receive a message on the mqueue
 * 
 * 2. to satisfy the third requirement, we not only have to be sure that B()
 *    does never fail the assertion 'y == 'z'', but also that B always reaches the line
 *    in which the assertion is present.
 *    Future lectures will cover the details of LTL model checking to verify the property :-)
 */
